import os
from subprocess import call, check_output, check_call
import subprocess

from .install import Installer
from .annotator import Annotator


class Pynnotator(object):
    """A class to annotate VCFs

    Attributes:
        vcf_file: a VCF file to be annotated
    """

    def __init__(self, vcf_file=False):
        """Return a Pynnotator object with a defined VCF file to be annotated."""
        self.vcf_file= vcf_file

    def install(self):
        print('Starting Installation...')
        installer = Installer()
        installer.install()
        
        print("Installation Finished with success!! \nNow try testing with the command: pynnotator test")
        
    def install_libs(self):
        print('Installing Libs...')
        installer = Installer()
        installer.install_libs()
    def install_requirements(self):
        print('Installing Libs...')
        installer = Installer()
        installer.install_requirements()
    def build(self):
        print('Building Databasets...')
        installer = Installer()
        installer.build_datasets()
    def annotate(self, vcf_file):
        print("Annotating VCF... %s" % (vcf_file))
        pynnotator = Annotator(vcf_file)
        pynnotator.run()

    def test(self):
        print('Testing Annotation...')
        path = '%s/tests' % (os.path.dirname(__file__))
        vcf_file = 'sample.1000.vcf'
        os.chdir(path)
        pynnotator = Annotator(vcf_file)
        pynnotator.run()
        os.chdir(path)

        command = 'grep -v "^#" ann_sample.1000/annotation.final.vcf > result.vcf'
        call(command, shell=True)

        # compare result with reference
        try:
            command = "zdiff annotation.vcf.gz result.vcf"
            diff_1 = check_output(command, shell=True)
        except subprocess.CalledProcessError as e:
            diff_1 = '1'

        try:    
            command = "zdiff annotation.v2.vcf.gz result.vcf"
            diff_2 = check_output(command, shell=True)#check_output
        except subprocess.CalledProcessError as e:
            diff_2 = '1'

        # print(diff_1, diff_2)
        if diff_1 == b'' or diff_2 == b'':
            print('Congratulations, The Python Annotation Framework is working as expected, Happy Annotation!!!')
 

        command = 'rm -rf ann_sample.1000 result.vcf'
        call(command, shell=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
from datetime import datetime
import os, shutil
import shlex, subprocess

from pynnotator import settings

toolname = 'merge'

class Merge(object):
    def __init__(self, vcffile=None):
        
        self.vcffile = vcffile

        self.filename = os.path.splitext(os.path.basename(str(vcffile)))[0]
        
        #create folder merge if it doesn't exists
        if not os.path.exists('merge'):
            os.makedirs('merge')
        #enter inside folder
        os.chdir('merge')
        
    
    def run(self):
        tstart = datetime.now()
        print(tstart, 'Starting merge: ', self.vcffile)
        
        self.merge()

        tend = datetime.now()
        annotation_time =  tend - tstart
        print(tend, 'Finished merge, it took: ', annotation_time)        

    #merge VCF Files with VariantAnnotator
    def merge(self):

        BASE_DIR = os.getcwd()
        print(BASE_DIR)

        #merge 5 vcfs: snpeff, vep, snpsift, hi_index and hgmd 
        # print('merge snpeff')
        #snpeff
        command = "java -Xmx%s -jar \
        %s/SnpSift.jar \
        annotate -info EFF -noId \
        ../snpeff/snpeff.output.vcf \
        ../sanity_check/checked.vcf \
        > snpeff.vcf \
        "% (settings.snpsift_merge_memory, settings.snpeff_dir)
        # print(command)

        p = subprocess.call(command, 
            cwd=os.getcwd(), 
            shell=True)
        # print 'snpeff'
        # print('merge vep')
        #vep
        command = "java -Xmx%s -jar \
        %s/SnpSift.jar \
        annotate -info CSQ -noId \
        ../vep/vep.output.sorted.vcf \
        snpeff.vcf \
        > snpeff.vep.vcf \
        "% (settings.snpsift_merge_memory, settings.snpeff_dir)

        p = subprocess.call(command, 
            cwd=os.getcwd(), 
            shell=True)
        # print('merge snpsift')
        #snpsift
        command = "java -Xmx%s -jar \
        %s/SnpSift.jar \
        annotate -info VARTYPE,SNP,MNP,INS,DEL,MIXED,HOM,HET -noId \
        ../snpsift/snpsift.final.vcf \
        snpeff.vep.vcf \
        > snpsift.snpeff.vep.vcf \
        "% (settings.snpsift_merge_memory, settings.snpeff_dir)

        p = subprocess.call(command, 
            cwd=os.getcwd(), 
            shell=True)

        # print('merge hi_index')
        #hi_index
        command = "java -Xmx%s -jar \
        %s/SnpSift.jar \
        annotate -info HI_INDEX -noId \
        ../hi_index/hi_index.vcf \
        snpsift.snpeff.vep.vcf \
        > hi_index.snpsift.snpeff.vep.vcf \
        "% (settings.snpsift_merge_memory, settings.snpeff_dir)

        p = subprocess.call(command, 
            cwd=os.getcwd(), 
            shell=True)

        # #hgmd
        # command = "java -Xmx40G -jar \
        # %s/SnpSift.jar \
        # annotate -info HGMD -noId \
        # ../hgmd/hgmd.vcf \
        # hi_index.snpsift.snpeff.vep.vcf \
        # > hgmd.hi_index.snpsift.snpeff.vep.vcf  \
        # "% (settings.snpeff_dir)


        # p = subprocess.call(command, 
        #     cwd=os.getcwd(), 
        #     shell=True)
        
        # print('merge pynnotator')
        #pynnotator - 1000genomes, dbsnp, clinvar, esp6500, ensembl
        command = "java -Xmx%s -jar \
        %s/SnpSift.jar \
        annotate -noId \
        ../pynnotator/pynnotator.vcf \
        hi_index.snpsift.snpeff.vep.vcf \
        > 1kgenomes.dbsnp.clinvar.esp.hi_index.snpsift.snpeff.vep.vcf  \
        " % (settings.snpsift_merge_memory, settings.snpeff_dir)

        p = subprocess.call(command, 
            cwd=os.getcwd(), 
            shell=True)

        # print   'func pred'
        
        # print('merge cadd_dann')
        #cadd_dann
        command = "java -Xmx%s -jar \
        %s/SnpSift.jar \
        annotate -noId -info dbNSFP_SIFT_score,dbNSFP_SIFT_converted_rankscore,dbNSFP_SIFT_pred,dbNSFP_Uniprot_acc_Polyphen2,dbNSFP_Uniprot_id_Polyphen2,dbNSFP_Uniprot_aapos_Polyphen2,dbNSFP_Polyphen2_HDIV_score,dbNSFP_Polyphen2_HDIV_rankscore,dbNSFP_Polyphen2_HDIV_pred,dbNSFP_Polyphen2_HVAR_score,dbNSFP_Polyphen2_HVAR_rankscore,dbNSFP_Polyphen2_HVAR_pred,dbNSFP_LRT_score,dbNSFP_LRT_converted_rankscore,dbNSFP_LRT_pred,dbNSFP_LRT_Omega,dbNSFP_MutationTaster_score,dbNSFP_MutationTaster_converted_rankscore,dbNSFP_MutationTaster_pred,dbNSFP_MutationTaster_model,dbNSFP_MutationTaster_AAE,dbNSFP_MutationAssessor_UniprotID,dbNSFP_MutationAssessor_variant,dbNSFP_MutationAssessor_score,dbNSFP_MutationAssessor_rankscore,dbNSFP_MutationAssessor_pred,dbNSFP_FATHMM_score,dbNSFP_FATHMM_converted_rankscore,dbNSFP_FATHMM_pred,dbNSFP_PROVEAN_score,dbNSFP_PROVEAN_converted_rankscore,dbNSFP_PROVEAN_pred,dbNSFP_Transcript_id_VEST3,dbNSFP_Transcript_var_VEST3,dbNSFP_VEST3_score,dbNSFP_VEST3_rankscore,dbNSFP_MetaSVM_score,dbNSFP_MetaSVM_rankscore,dbNSFP_MetaSVM_pred,dbNSFP_MetaLR_score,dbNSFP_MetaLR_rankscore,dbNSFP_MetaLR_pred,dbNSFP_Reliability_index,dbNSFP_CADD_raw,dbNSFP_CADD_raw_rankscore,dbNSFP_CADD_phred,dbNSFP_DANN_score,dbNSFP_DANN_rankscore,dbNSFP_fathmm,dbNSFP_fathmm,dbNSFP_fathmm,dbNSFP_fathmm,dbNSFP_Eigen,dbNSFP_Eigen,dbNSFP_Eigen,dbNSFP_Eigen,dbNSFP_Eigen,dbNSFP_GenoCanyon_score,dbNSFP_GenoCanyon_score_rankscore,dbNSFP_integrated_fitCons_score,dbNSFP_integrated_fitCons_rankscore,dbNSFP_integrated_confidence_value,dbNSFP_GM12878_fitCons_score,dbNSFP_GM12878_fitCons_rankscore,dbNSFP_GM12878_confidence_value,dbNSFP_H1,dbNSFP_H1,dbNSFP_H1,dbNSFP_HUVEC_fitCons_score,dbNSFP_HUVEC_fitCons_rankscore,dbNSFP_HUVEC_confidence_value,dbNSFP_GERP,dbNSFP_GERP,dbNSFP_GERP,dbNSFP_phyloP100way_vertebrate,dbNSFP_phyloP100way_vertebrate_rankscore,dbNSFP_phyloP20way_mammalian,dbNSFP_phyloP20way_mammalian_rankscore,dbNSFP_phastCons100way_vertebrate,dbNSFP_phastCons100way_vertebrate_rankscore,dbNSFP_phastCons20way_mammalian,dbNSFP_phastCons20way_mammalian_rankscore,dbNSFP_SiPhy_29way_pi,dbNSFP_SiPhy_29way_logOdds,dbNSFP_SiPhy_29way_logOdds_rankscore,dbNSFP_clinvar_rs,dbNSFP_clinvar_clnsig,dbNSFP_clinvar_trait,dbNSFP_clinvar_golden_stars \
        ../func_pred/func_pred.vcf \
        1kgenomes.dbsnp.clinvar.esp.hi_index.snpsift.snpeff.vep.vcf \
        > func_pred.1kgenomes.dbsnp.clinvar.esp.hi_index.snpsift.snpeff.vep.vcf  \
        "% (settings.snpsift_merge_memory, settings.snpeff_dir)


        p = subprocess.call(command, 
            cwd=os.getcwd(), 
            shell=True)

        # print('merge dbsnp')
        #dbsnp
        command = "java -Xmx%s -jar \
        %s/SnpSift.jar \
        annotate -id \
        %s \
        func_pred.1kgenomes.dbsnp.clinvar.esp.hi_index.snpsift.snpeff.vep.vcf \
        > annotation.final.vcf  \
        "% (settings.snpsift_merge_memory, settings.snpeff_dir,settings.dbsnp)


        p = subprocess.call(command, 
            cwd=os.getcwd(), 
            shell=True)

        if p == 0:
            print('This vcf was succesfully merged')
        else:
            print('Sorry this vcf could not be merged')


if  __name__ == '__main__' :

    parser = argparse.ArgumentParser(description='MERGE all VCF Files from previous methods.')
    parser.add_argument('-i', dest='vcffile', required=True, metavar='example.vcf', help='a VCF file to be annotated')
    args = parser.parse_args()

    merge = Merge(args.vcffile)
    merge.run()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import argparse
import os
import sys

import time
from datetime import datetime

import csv
import shutil
import shlex, subprocess

from threading import Thread

from pynnotator import settings

from .helpers import validator, sanity_check, snpeff, vep, hi_index, snpsift, vcf_annotator, func_pred, merge

class Annotator(object):

    is_validated = False

    def __init__(self, vcf_file=None):
        
        self.filename = os.path.splitext(os.path.basename(str(vcf_file)))[0]

        # create a folder for the annotation if it doesn't exists, 
        # or delete and create if the folder already exists
        ann_name = "ann_%s" % (self.filename)

        self.vcf_file = os.path.abspath(vcf_file)
        
        if os.path.exists(ann_name):
            shutil.rmtree(ann_name)

        os.makedirs(ann_name)
        os.chdir(ann_name)
        #copy vcf to ann folder and rewrite self.vcf_file
        
        print('vcf_file', self.vcf_file)
        # os.path.basename(str(vcf_file))
        
        # os.makedirs('log')
        # os.makedirs('reports')




    def run(self):
        
        tstart = datetime.now()

        #run vcftools vcf-validator on this file 
        validator = Thread(target=self.validator)
        validator.start()
        #wait the process to finish before continue to the next steps 
        validator.join()

        # #Check if thew vcf file is validated
        if(self.is_validated):#
            
            threads = []

            sanitycheck = Thread(target=self.sanitycheck)
            sanitycheck.start()
            # #wait till finish to continue
            sanitycheck.join()

            self.vcf_file = 'sanity_check/checked.vcf'

            snpeff = Thread(target=self.snpeff)
            threads.append(snpeff)

            vep = Thread(target=self.vep)
            threads.append(vep)

            hi_index = Thread(target=self.hi_index)
            threads.append(hi_index)

            snpsift = Thread(target=self.snpsift)
            threads.append(snpsift)

            vcf_annotator = Thread(target=self.vcf_annotator)
            threads.append(vcf_annotator)

            func_pred = Thread(target=self.func_pred) #took 0:17:40.699580
            threads.append(func_pred)

            #execute all tasks in parallel
            for thread in threads:
                thread.start()
                #thread.join()#this option to make it serial
            
            for thread in threads:
                thread.join()

            merge = Thread(target=self.merge)
            merge.start()
            # #wait till finish to continue
            merge.join()
            
            print("Annotation Completed!")
            tend = datetime.now()
            execution_time = tend -  tstart
            # logging.info('Finished Annotation, it took %s' % (execution_time))
            print('Finished Annotation, it took %s' % (execution_time))

        #     print 'Delete ann'
        #     print os.getcwd()
            #remove folder
            # if options.delete_ann:
            #     command = 'rm -rf ../ann'
            #     self.shell(command)


    def log_message(self, message):
        print(message)
        # logging.info(message)
    def shell(self, command):
        # logging.info('Running Command %s' % (command))
        # logging.info('OS CWD: %s' % (os.getcwd()))
        print('Running Command %s' % (command))
        # print 'OS CWD: %s' % (os.getcwd())
        
        try:
            p = subprocess.check_output(command, 
            cwd=os.getcwd(), 
            shell=True)
            # logging.info('Command Output: %s' % (p))
        except subprocess.CalledProcessError as e:
            print('CalledProcessError:', e)

    #if can validate return True else return False
    def validator(self):
        """
        VCF Validator

        """
        tstart = datetime.now()
        v = validator.Validator(self.vcf_file)
        std = v.run()
        if std == 0:
            self.is_validated = True

        tend = datetime.now()
        execution_time = tend -  tstart
        
    def sanitycheck(self):
        """
        Search and Remove variants with [0/0, ./.]
        Search and Replace chr from the beggining of the chromossomes to get positionning.    
        Sort VCF by 1...22, X, Y, MT and nothing else
        #Discard other variants
        """
        #logging.info('Starting Sanity Check...')
        tstart = datetime.now()

        # command = 'python %s/sanity_check.py -i %s' % (scripts_dir, self.vcf_file)
        # self.shell(command)

        sc = sanity_check.Sanity_check(self.vcf_file)
        std = sc.run()
        
        tend = datetime.now()
        execution_time = tend -  tstart
        #logging.info('Finished Sanity Check, it took %s' % (execution_time))

    def snpeff(self):
        """
        Annotation with snpEff 
        """        
        #calculate time thread took to finish
        #logging.info('Starting snpEff')
        tstart = datetime.now()

        se = snpeff.Snpeff(self.vcf_file)
        std = se.run()

        tend = datetime.now()
        execution_time = tend -  tstart
        #logging.info('Finished snpEff, it took %s' % (execution_time))
        print('Finished snpEff, it took %s' % (execution_time))

        #subprocess.call(args)

    def vep(self):
        """VEP"""
        
        #calculate time thread took to finish
        #logging.info('Starting VEP ')
        tstart = datetime.now()

        vep_obj = vep.Vep(self.vcf_file)
        std = vep_obj.run()
        
        # command = 'python %s/vep.py -i sanity_check/checked.vcf' % (scripts_dir)
        # self.shell(command)

        # command = 'mv vep/vep.log log/'
        # os.system(command)
        
        # command = 'mv vep/vep.output.vcf_summary.html reports/vep_summary.html'
        # os.system(command)

        tend = datetime.now()
        execution_time = tend -  tstart
        #logging.info('Finished VEP, it took %s' % (execution_time))
        print('Finished VEP, it took %s' % (execution_time))
    
   
    def hi_index(self):
        """Hi Index """

        
        #calculate time thread took to finish
        #logging.info('Starting HI score')
        tstart = datetime.now()
        
        # command = 'python %s/hi_index.py -i sanity_check/checked.vcf' % (scripts_dir)
        # self.shell(command)

        hi = hi_index.HI_Index(self.vcf_file)
        hi.run()

        tend = datetime.now()
        execution_time = tend -  tstart
        #logging.info('Finished HI Score, it took %s' % (execution_time))
        print('Finished Hi Index, it took %s' % (execution_time))

    def hgmd(self):
        """Hi Index """

        
        #calculate time thread took to finish
        #logging.info('Starting HI score')
        tstart = datetime.now()
        
        command = 'python %s/hgmd.py -i sanity_check/checked.vcf' % (scripts_dir)
        self.shell(command)


        tend = datetime.now()
        execution_time = tend -  tstart
        #logging.info('Finished HI Score, it took %s' % (execution_time))
        print('Finished HGMD, it took %s' % (execution_time))
        
   
    def merge(self):

        print("Merging all VCF Files...")
        t_merge_start = datetime.now()
        # # #merge VCF Files
        # command = 'python %s/merge.py -i sanity_check/checked.vcf' % (scripts_dir)
        # self.shell(command)
        mg = merge.Merge(self.vcf_file)
        mg.run()

        t_merge_end = datetime.now()
        execution_time = t_merge_end -  t_merge_start

        # logging.info('Finished Merging VCF, it took %s' % (execution_time))
        print('Finished Merging VCF, it took %s' % (execution_time))
        
        #compress annotation file to save space
        # command = '%s/bgzip merge/annotation.final.vcf' % (tabix_path)
        # os.system(command)
        # print('before merge', os.getcwd())
        #move final file one up and delete folder!
        command = 'mv annotation.final.vcf ../'
        os.system(command)

    def snpsift(self):
        """SnpSift"""
        
        tstart = datetime.now()
        
        # command = 'python %s/snpsift.py -i sanity_check/checked.vcf 2>log/snpsift.log' % (scripts_dir)
        # self.shell(command)

        ss = snpsift.SnpSift(self.vcf_file)
        ss.run()

        
        tend = datetime.now()
        execution_time = tend -  tstart
        #logging.info('Finished annovar, it took %s' % (execution_time))
        print('Finished snpsift, it took %s' % (execution_time))

    def vcf_annotator(self):
        """Vcf annotator"""
        
        tstart = datetime.now()

        #python ../scripts/annotate_vcfs.py -i mm13173_14.ug.target1.vcf -r 1000genomes dbsnp138 clinvar esp6500 -a ../data/1000genomes/ALL.wgs.integrated_phase1_v3.20101123.snps_indels_sv.sites.vcf.gz ../data/dbsnp138/00-All.vcf.gz ../data/dbsnp138/clinvar_00-latest.vcf.gz ../data/ESP6500/ESP6500.vcf.gz

        # command = 'python %s/vcf_annotator_parallel.py -n %s -i sanity_check/checked.vcf -r 1000genomes dbsnp clinvar esp6500 -a %s %s %s %s 2>log/pynnotator.log' % (scripts_dir, pynnotator_cores, genomes1k, dbsnp, clinvar, esp)
        # self.shell(command)

        resources = "genomes1k dbsnp clinvar esp6500 ensembl_phen ensembl_clin"#    
        resources = resources.split(' ')
        annfiles = [
        "%s/1000genomes/%s" % (settings.data_dir, settings.genomes1k_file),
        "%s/dbsnp/%s" % (settings.data_dir, settings.dbsnp_file),
        "%s/dbsnp/%s" % (settings.data_dir, settings.clinvar_file),
        "%s/esp6500/%s" % (settings.data_dir, settings.esp_final_file),
        "%s/ensembl/%s" % (settings.data_dir, settings.ensembl_phenotype_file),
        "%s/ensembl/%s" % (settings.data_dir, settings.ensembl_clinically_file),
        ]
        #
 


        # annfiles = " ".join(annfiles)

        # annfiles = ["%s/1000genomes/%s" % (settings.data_dir, settings.genomes1k_file)]

        annotator_obj =  vcf_annotator.VCF_Annotator(self.vcf_file, annfiles, resources, settings.vcf_annotator_cores)

        annotator_obj.run()
        
        tend = datetime.now()
        execution_time = tend -  tstart
        #logging.info('Finished annovar, it took %s' % (execution_time))
        print('Finished vcf_annotator, it took %s' % (execution_time))
    def func_pred(self):
        """func_pred"""
        
        tstart = datetime.now()

        #python ../scripts/annotate_vcfs.py -i mm13173_14.ug.target1.vcf -r 1000genomes dbsnp138 clinvar esp6500 -a ../data/1000genomes/ALL.wgs.integrated_phase1_v3.20101123.snps_indels_sv.sites.vcf.gz ../data/dbsnp138/00-All.vcf.gz ../data/dbsnp138/clinvar_00-latest.vcf.gz ../data/ESP6500/ESP6500.vcf.gz

        # command = 'python %s/cadd_dann.py -n %s -i sanity_check/checked.vcf 2>log/cadd_dann.log' % (scripts_dir, cadd_vest_cores)
        # self.shell(command)
        fp = func_pred.FUNC_PRED_Annotator(self.vcf_file, settings.func_pred_cores)
        fp.run()
        
        tend = datetime.now()
        execution_time = tend -  tstart
        #logging.info('Finished annovar, it took %s' % (execution_time))
        print('Finished Func Pred, it took %s' % (execution_time))



if __name__=="__main__":
    
    parser = argparse.ArgumentParser(description='Annotate a VCF File with Annotator.')
    parser.add_argument('-i', dest='vcf_file', required=True, metavar='example.vcf', help='a VCF file to be annotated')
    args = parser.parse_args()

    a = Annotator(args.vcf_file)
    a.run()

